//
//  BWAppDelegate.m
//  Lineinator
//
//  Created by Mark Dalrymple on 8/10/12.
//  Copyright (c) 2012 Mark Dalrymple. All rights reserved.
//

#import "BWAppDelegate.h"
#import <mach/mach_time.h>

CGFloat BNRTimeBlock (void (^block)(void)) {
    mach_timebase_info_data_t info;
    if (mach_timebase_info(&info) != KERN_SUCCESS) return -1.0;

    uint64_t start = mach_absolute_time ();
    block ();
    uint64_t end = mach_absolute_time ();
    uint64_t elapsed = end - start;

    uint64_t nanos = elapsed * info.numer / info.denom;
    return (CGFloat)nanos / NSEC_PER_SEC;

} // BNRTimeBlock


#define COUNT 400

@interface BWLineView : NSView {
    CGPoint _points[COUNT];

    BOOL _singlePath;
}
@end // BWLineView



@implementation BWAppDelegate
@end


@implementation BWLineView

- (void) doPoints {
    CGRect bounds = self.bounds;
    
    for (int i = 0; i < COUNT; i++) {
        _points[i] = CGPointMake (random() % (int)bounds.size.width,
                                  random() % (int)bounds.size.height);
    }

} // doPoints


- (id) initWithFrame: (CGRect) frame {
    if ((self = [super initWithFrame: frame])) {
        [self doPoints];
        _singlePath = YES;
    }
    return self;

} // initWithFrame


- (void) drawRect: (CGRect) rect {

    CGRect bounds = self.bounds;
    [[NSColor whiteColor] set];
    NSRectFill (bounds);

    CGFloat duration = BNRTimeBlock (^{
            if (_singlePath) {
                NSBezierPath *path = [NSBezierPath bezierPath];
                [path moveToPoint: _points[0]];
                
                for (int i = 1; i < COUNT; i++) {
                    [path lineToPoint: _points[i]];
                }
                
                path.lineWidth = 7.0;
                
                NSColor *color = [NSColor colorWithDeviceRed: 0.0
                                          green: 1.0
                                          blue: 0.0
                                          alpha: 0.3];
                [color set];
                [path stroke];
            } else {
                NSColor *color = [NSColor colorWithDeviceRed: 0.0
                                          green: 1.0
                                          blue: 0.0
                                          alpha: 0.3];
                [color set];
                
                [NSBezierPath setDefaultLineWidth: 7.0];
                
                for (int i = 0; i < COUNT - 1; i++) {
                    [NSBezierPath strokeLineFromPoint: _points[i]
                                  toPoint: _points[i + 1]];
                }
            }
        });
    NSLog (@"took %f to draw %d points", duration, COUNT);

    [[NSColor blackColor] set];
    NSFrameRect (bounds);

} // drawRect


- (void) setFrame: (CGRect) frame {
    [super setFrame: frame];
    [self doPoints];
} // setFrame


- (IBAction) doSinglePath: (NSButton *) sender {
    _singlePath = sender.state == NSOnState;
    [self setNeedsDisplay: YES];
} // doSinglePath


@end // BWLineView
